/**
 * Soluzione che risponde alle query in O(N+Q) usando LCA offline (versione
 * pulita)
 */
#include <bits/stdc++.h>
using namespace std;
#define DELETED -1

#ifdef EVAL
ifstream in("input.txt");
ofstream out("output.txt");
#else
istream& in(cin);
ostream& out(cout);
#endif

struct Edge {
    int to,    // nodo di destinazione
        index; // indice dell'arco
    Edge(int _to, int _index) : to(_to), index(_index) {}
};

/**
 * MFset che supporta anche rimozione di un elemento da un set e l'iterazione
 * sugli elementi di un set
 */
struct MFset {
    vector<int> parent, rank, size;
    vector<unordered_set<int>> children;
    int n, m;

    void init(int _n) {
        n = _n;
        m = n + n;
        parent.resize(n + n);
        rank.resize(n + n, 0);
        size.resize(n + n, 0);
        children.resize(n);
        for (int i = 0; i < n; i++)
            parent[i] = i + n;

        for (int i = n; i < n + n; i++) {
            parent[i] = i;
            rank[i] = 1;
            size[i] = 1;
            children[i - n].insert(i - n);
        }
    }

    void delete_chain(int x) {
        while (parent[x] != x && (x < n || children[x - n].empty())) {
            // if (x >= n) {
            //     children[x - n] = {};
            //     while (x == m - 1) {
            //         m--;
            //         children.pop_back();
            //     }
            // }
            int px = parent[x];
            // assert(px >= n);
            children[px - n].erase(x);
            parent[x] = DELETED;
            x = px;
        }
    }

    void link_to(int x, int new_parent) {
        if (parent[x] != new_parent) {
            delete_chain(x);
            // assert(new_parent >= n);
            children[new_parent - n].insert(x);
            parent[x] = new_parent;
        }
    }

    int find(int x) {
        if (parent[x] == DELETED)
            return DELETED;
        if (parent[x] != x)
            link_to(x, find(parent[x]));
        return parent[x];
    }

    void merge(int x, int y) {
        int px = find(x), py = find(y);
        if (px != py) {
            if (rank[px] < rank[py]) {
                link_to(px, py);
                size[py] += size[px];
            } else {
                link_to(py, px);
                size[px] += size[py];
                rank[px] += rank[px] == rank[py];
            }
        }
    }

    bool remove(int x, int set_id) {
        set_id = find(set_id);
        if (find(x) != set_id)
            return false;
        delete_chain(x);
        size[set_id]--;
        return true;
    }

    /**
     * Rimuove dal set corrente [set_id] e da other[other_id] gli elementi in
     * comune.
     */
    void remove_from(int set_id, MFset& other, int other_id) {
        set_id = find(set_id);
        other_id = other.find(other_id);
        stack<int> other_elements;
        other_elements.push(other_id);
        while (!other_elements.empty()) {
            int e = other_elements.top();
            other_elements.pop();
            if (e < other.n) {
                // appiattisco l'altro set per velocizzare la rimozione dopo
                if (parent[e] != other_id)
                    other.link_to(e, other_id);
                if (remove(e, set_id))
                    assert(other.remove(e, other_id));
            } else
                for (int c : other.children[e - other.n])
                    other_elements.push(c);
        }
    }

    int get_size(int set_id) { return set_id == -1 ? 0 : size[find(set_id)]; }
};

struct Tree {
    vector<vector<Edge>> tree;
    vector<bool> visited;
    MFset st_queries, en_queries;
    vector<int> st_q_set, en_q_set;

    void init(int n) { tree.resize(n); }

    void add_undirected_edge(int a, int b, int id) {
        tree[a].push_back(Edge(b, id));
        tree[b].push_back(Edge(a, id));
    }

    bool dfs_answer_queries(int i, vector<pair<int, int>>& edges_orientation,
                            int& conflict_edge) {
        visited[i] = true;
        int parent_edge = -1;

        for (Edge& e : tree[i]) {
            int c = e.to;
            if (!visited[c]) {
                if (!dfs_answer_queries(c, edges_orientation, conflict_edge))
                    return false;
                if (st_q_set[c] != -1) {
                    if (st_q_set[i] != -1)
                        st_queries.merge(st_q_set[i], st_q_set[c]);
                    st_q_set[i] = st_queries.find(st_q_set[c]);
                    edges_orientation[e.index] = make_pair(c, i);
                }
                if (en_q_set[c] != -1) {
                    if (en_q_set[i] != -1)
                        en_queries.merge(en_q_set[i], en_q_set[c]);
                    en_q_set[i] = en_queries.find(en_q_set[c]);
                    edges_orientation[e.index] = make_pair(i, c);
                }
            } else
                parent_edge = e.index;
        }

        int st_index = st_q_set[i], en_index = en_q_set[i];
        if (st_index == -1 || en_index == -1)
            return true;
        bool st_smaller =
            st_queries.get_size(st_index) <= en_queries.get_size(en_index);

        MFset& smaller = st_smaller ? st_queries : en_queries;
        MFset& bigger = st_smaller ? en_queries : st_queries;
        int s_i = st_smaller ? st_index : en_index;
        int b_i = st_smaller ? en_index : st_index;

        int bsb = bigger.get_size(b_i);
        int ssb = smaller.get_size(s_i);

        bigger.remove_from(b_i, smaller, s_i);
        if (st_queries.get_size(st_q_set[i]) == 0)
            st_q_set[i] = -1;
        if (en_queries.get_size(en_q_set[i]) == 0)
            en_q_set[i] = -1;

        if (smaller.get_size(s_i) > 0)
            conflict_edge = parent_edge;

        return smaller.get_size(s_i) == 0;
    }

    bool answer_queries(vector<pair<int, int>>& queries,
                        vector<pair<int, int>>& edges_orientation,
                        int& conflict_edge) {
        st_queries.init(queries.size());
        en_queries.init(queries.size());
        st_q_set.assign(tree.size(), -1);
        en_q_set.assign(tree.size(), -1);
        visited.assign(tree.size(), false);
        for (int i = 0; i < queries.size(); i++) {
            auto q = queries[i];
            if (st_q_set[q.first] != -1)
                st_queries.merge(st_q_set[q.first], i);
            st_q_set[q.first] = st_queries.find(i);
            if (en_q_set[q.second] != -1)
                en_queries.merge(en_q_set[q.second], i);
            en_q_set[q.second] = en_queries.find(i);
        }
        return dfs_answer_queries(0, edges_orientation, conflict_edge);
    }
};

struct Graph {
    vector<vector<Edge>> g;
    vector<bool> visited;
    vector<int> bfc;
    vector<bool> is_bridge;
    vector<int> bridges;
    vector<pair<int, int>> edges;

    Graph(int n) : g(n) {}

    void add_undirected_edge(int a, int b) {
        g[a].push_back(Edge(b, edges.size()));
        g[b].push_back(Edge(a, edges.size()));
        is_bridge.push_back(false);
        edges.push_back(make_pair(a, b));
    }

    void dfs_bridges(int v, vector<int>& tin, vector<int>& low, int& timer,
                     int p = -1) {
        visited[v] = true;
        tin[v] = low[v] = timer++;
        for (Edge& e : g[v]) {
            int to = e.to;
            if (to == p)
                continue;
            if (visited[to]) {
                low[v] = min(low[v], tin[to]);
            } else {
                dfs_bridges(to, tin, low, timer, v);
                low[v] = min(low[v], low[to]);
                // assert(edges_count[make_pair(min(v, to), max(v, to))] >= 1);
                if (low[to] > tin[v]) {
                    is_bridge[e.index] = true;
                    bridges.push_back(e.index);
                }
            }
        }
    }

    /**
     * Source: https://cp-algorithms.com/graph/bridge-searching.html
     */
    void find_bridges() {
        visited.assign(g.size(), false);
        vector<int> tin(g.size(), -1), low(g.size(), -1);
        int timer = 0;
        for (int i = 0; i < g.size(); ++i) {
            if (!visited[i])
                dfs_bridges(i, tin, low, timer);
        }
    }

    /**
     * Visito la bridge-free-component
     */
    void visit_bfc(int v, int counter) {
        bfc[v] = counter;
        // cerr << "BFC[" << v << "] = " << counter << endl;
        for (Edge& e : g[v]) {
            int to = e.to;
            if (bfc[to] == -1 && !is_bridge[e.index])
                visit_bfc(to, counter);
        }
    }

    /**
     * Costruisce il bridge tree del grafo
     */
    void get_bridge_tree(Tree& t) {
        find_bridges();
        int counter = 0;
        bfc.assign(g.size(), -1);
        for (int i = 0; i < g.size(); i++)
            if (bfc[i] == -1)
                visit_bfc(i, counter++);
        t.init(counter);
        for (int i = 0; i < bridges.size(); i++) {
            int b = bridges[i];
            // assert(bfc[edges[b].first] != bfc[edges[b].second]);
            t.add_undirected_edge(bfc[edges[b].first], bfc[edges[b].second], i);
        }
    }

    void free() {
        is_bridge.clear();
        g.clear();
        visited.clear();
    }
};

/**
 * Data un'orientazione degli archi del bridge tree, orienta i bridge
 * corrispondenti
 */
void get_bridges_orientation(vector<pair<int, int>>& tree_edges, Graph& g,
                             vector<pair<int, int>>& oriented_bridges) {

    for (int i = 0; i < g.bridges.size(); i++) {
        pair<int, int>& graph_edge = g.edges[g.bridges[i]];
        pair<int, int>& tree_edge = tree_edges[i];
        if (tree_edge != make_pair(-1, -1)) {
            if (tree_edge.first != g.bfc[graph_edge.first])
                swap(graph_edge.first, graph_edge.second);
            // assert(tree_edge == make_pair(g.bfc[graph_edge.first],
            // g.bfc[graph_edge.second]));
            oriented_bridges.push_back(graph_edge);
        }
    }
}

/**
 * Traduce query sul grafo originale in query sul bridge tree
 */
void translate_queries(vector<pair<int, int>>& queries, vector<int>& bfc) {
    for (auto& q : queries) {
        q.first = bfc[q.first];
        q.second = bfc[q.second];
    }
}

int main() {
    int n, m, q;
    int a, b;
    in >> n >> m >> q;
    Graph g(n);
    for (int i = 0; i < m; i++) {
        in >> a >> b;
        g.add_undirected_edge(a, b);
    }
    vector<pair<int, int>> queries;
    for (int i = 0; i < q; i++) {
        in >> a >> b;
        queries.push_back(make_pair(a, b));
    }
    // costruisco il bridge tree
    Tree bridge_tree;
    g.get_bridge_tree(bridge_tree);
    g.free();
    // trovo un'orientazione agli archi dell'albero
    vector<pair<int, int>> tree_edges_orientation(g.bridges.size(),
                                                  make_pair(-1, -1));
    int conflict_edge = -1;
    translate_queries(queries, g.bfc);

    if (bridge_tree.answer_queries(queries, tree_edges_orientation,
                                   conflict_edge)) {
        out << "1" << endl;
        // riconverto gli archi dell'albero nei bridges originali
        vector<pair<int, int>> oriented_bridges;
        get_bridges_orientation(tree_edges_orientation, g, oriented_bridges);
        out << oriented_bridges.size() << endl;
        for (auto& bridge : oriented_bridges)
            out << bridge.first << " " << bridge.second << endl;

    } else {
        out << "0" << endl;
        pair<int, int> conflict = g.edges[g.bridges[conflict_edge]];
        out << conflict.first << " " << conflict.second << endl;
    }
}
