#include <bits/stdc++.h>
using namespace std;

#ifdef EVAL
ifstream in("input.txt");
ofstream out("output.txt");
#else
istream& in(cin);
ostream& out(cout);
#endif

typedef pair<int, int> ii;
struct Edge {
    int to,    // nodo di destinazione
        index; // indice dell'arco
    Edge(int _to, int _index) : to(_to), index(_index) {}
};

struct RequestMinimumQuery {
    vector<int> log_table;
    vector<vector<int>> sparse_table;
    RequestMinimumQuery() {}

    void populate(vector<int>& v) {
        log_table.assign(v.size() + 1, 0);
        for (int i = 2; i < log_table.size(); i++)
            log_table[i] = log_table[i / 2] + 1;
        sparse_table.assign(log_table.back() + 1, vector<int>(v.size()));
        sparse_table[0] = v;
        for (int i = 1; i < sparse_table.size(); i++) {
            for (int j = 0; j + (1 << i) <= v.size(); j++) {
                sparse_table[i][j] =
                    min(sparse_table[i - 1][j],
                        sparse_table[i - 1][j + (1 << (i - 1))]);
            }
        }
    }
    int minimum(int left, int right) {
        int log = log_table[right - left];
        return min(sparse_table[log][left],
                   sparse_table[log][right - (1 << log)]);
    }
};

/**
 * Traduce query sul grafo originale in query sul bridge tree
 */
void translate_queries(vector<pair<int, int>>& queries, vector<int>& bfc) {
    for (auto& q : queries) {
        q.first = bfc[q.first];
        q.second = bfc[q.second];
    }
}

struct Tree {
    int root;
    vector<vector<Edge>> tree;
    unordered_map<int, int> root_dist;
    vector<int> euler_tour;
    unordered_map<int, int> first_encounter;
    RequestMinimumQuery rmq;
    vector<int> new_to_old;
    vector<int> count_a, // quante volte il nodo i è source
        count_b,         // quante volte il nodo i è dest
        count_c;         // quante volte il nodo i è lca

    void init(int n) {
        root = 0;
        tree.resize(n);
    }

    void add_undirected_edge(int a, int b, int id) {
        tree[a].push_back(Edge(b, id));
        tree[b].push_back(Edge(a, id));
    }

    void precompute() {
        root_dist = distance_from_root(root);
        euler(root, -1);
        rmq.populate(euler_tour);
    }

    int get_lca(int a, int b) {
        int source = first_encounter[a];
        int target = first_encounter[b];
        if (source > target)
            swap(source, target);
        int new_index = rmq.minimum(source, target + 1);
        return new_to_old[new_index];
    }

    void euler(int source, int parent) {
        int new_index = new_to_old.size();
        new_to_old.push_back(source);
        first_encounter[source] = euler_tour.size();
        euler_tour.push_back(new_index);
        for (Edge& e : tree[source]) {
            int to = e.to;
            if (to != parent) {
                euler(to, source);
                euler_tour.push_back(new_index);
            }
        }
    }

    unordered_map<int, int> distance_from_root(int root) {
        queue<int> q;
        unordered_map<int, int> distance;
        unordered_map<int, bool> visited;
        int level = 0;
        q.push(root);
        distance[root] = level;
        visited[root] = true;
        while (!q.empty()) {
            int size = q.size();
            level++;
            for (int i = 0; i < size; i++) {
                int current = q.front();
                q.pop();
                for (Edge& e : tree[current]) {
                    int to = e.to;
                    if (!visited[to]) { // if not parent
                        q.push(to);
                        visited[to] = true;
                        distance[to] = level;
                    }
                }
            }
        }
        return distance;
    }

    /**
     * Orienta gli archi dell'albero e trova un eventuale arco in conflitto.
     */
    bool dfs_orient_edges(int v, int parent,
                          vector<pair<int, int>>& edges_orientation,
                          int& conflict_edge) {
        int parent_edge = -1;
        for (Edge& e : tree[v]) {
            int u = e.to;
            if (u != parent) {
                if (!dfs_orient_edges(u, v, edges_orientation, conflict_edge))
                    return false;
                count_a[v] += count_a[u];
                count_b[v] += count_b[u];
                count_c[v] += count_c[u];
            } else
                parent_edge = e.index;
        }

        // oriento gli archi
        if (count_a[v] > count_c[v] && count_b[v] > count_c[v]) {
            assert(parent_edge != -1);
            conflict_edge = parent_edge;
            return false;
        }
        assert(count_a[v] >= count_c[v] && count_b[v] >= count_c[v]);
        if (parent_edge != -1) {
            if (count_a[v] > count_c[v])
                edges_orientation[parent_edge] = make_pair(v, parent);
            else if (count_b[v] > count_c[v])
                edges_orientation[parent_edge] = make_pair(parent, v);
            else
                assert(count_a[v] == count_b[v] && count_b[v] == count_c[v]);
        }
        return true;
    }

    /**
     * Orienta gli archi dell'albero per rispondere alle query
     * Usa LCA con RMQ.
     */
    bool answer_queries(vector<pair<int, int>>& queries,
                        vector<pair<int, int>>& edges_orientation,
                        int& conflict_edge) {
        vector<vector<int>> node_queries(tree.size());
        count_a.assign(tree.size(), 0);
        count_b.assign(tree.size(), 0);
        count_c.assign(tree.size(), 0);
        precompute();
        for (auto& q : queries) {
            if (q.first != q.second) {
                count_a[q.first]++;
                count_b[q.second]++;
                count_c[get_lca(q.first, q.second)]++;
            }
        }

        return dfs_orient_edges(root, -1, edges_orientation, conflict_edge);
    }
};

struct Graph {
    vector<vector<Edge>> g;
    vector<bool> visited;
    vector<int> bfc;
    vector<bool> is_bridge;
    vector<int> bridges;
    vector<pair<int, int>> edges;

    Graph(int n) : g(n) {}

    void add_undirected_edge(int a, int b) {
        g[a].push_back(Edge(b, edges.size()));
        g[b].push_back(Edge(a, edges.size()));
        is_bridge.push_back(false);
        edges.push_back(make_pair(a, b));
    }

    void dfs_bridges(int v, vector<int>& tin, vector<int>& low, int& timer,
                     int p = -1) {
        visited[v] = true;
        tin[v] = low[v] = timer++;
        for (Edge& e : g[v]) {
            int to = e.to;
            if (to == p)
                continue;
            if (visited[to]) {
                low[v] = min(low[v], tin[to]);
            } else {
                dfs_bridges(to, tin, low, timer, v);
                low[v] = min(low[v], low[to]);
                // assert(edges_count[make_pair(min(v, to), max(v, to))] >= 1);
                if (low[to] > tin[v]) {
                    is_bridge[e.index] = true;
                    bridges.push_back(e.index);
                }
            }
        }
    }

    /**
     * Source: https://cp-algorithms.com/graph/bridge-searching.html
     */
    void find_bridges() {
        visited.assign(g.size(), false);
        vector<int> tin(g.size(), -1), low(g.size(), -1);
        int timer = 0;
        for (int i = 0; i < g.size(); ++i) {
            if (!visited[i])
                dfs_bridges(i, tin, low, timer);
        }
    }

    /**
     * Visito la bridge-free-component
     */
    void visit_bfc(int v, int counter) {
        bfc[v] = counter;
        // cerr << "BFC[" << v << "] = " << counter << endl;
        for (Edge& e : g[v]) {
            int to = e.to;
            if (bfc[to] == -1 && !is_bridge[e.index])
                visit_bfc(to, counter);
        }
    }

    /**
     * Costruisce il bridge tree del grafo
     */
    void get_bridge_tree(Tree& t) {
        find_bridges();
        int counter = 0;
        bfc.assign(g.size(), -1);
        for (int i = 0; i < g.size(); i++)
            if (bfc[i] == -1)
                visit_bfc(i, counter++);
        t.init(counter);
        for (int i = 0; i < bridges.size(); i++) {
            int b = bridges[i];
            // assert(bfc[edges[b].first] != bfc[edges[b].second]);
            t.add_undirected_edge(bfc[edges[b].first], bfc[edges[b].second], i);
        }
    }
};

/**
 * Data un'orientazione degli archi del bridge tree, orienta i bridge
 * corrispondenti
 */
void get_bridges_orientation(vector<pair<int, int>>& tree_edges, Graph& g,
                             vector<pair<int, int>>& oriented_bridges) {

    for (int i = 0; i < g.bridges.size(); i++) {
        pair<int, int>& graph_edge = g.edges[g.bridges[i]];
        pair<int, int>& tree_edge = tree_edges[i];
        if (tree_edge != make_pair(-1, -1)) {
            if (tree_edge.first != g.bfc[graph_edge.first])
                swap(graph_edge.first, graph_edge.second);
            assert(tree_edge == make_pair(g.bfc[graph_edge.first],
                                          g.bfc[graph_edge.second]));
            oriented_bridges.push_back(graph_edge);
        }
    }
}

int main() {
    int n, m, q;
    int a, b;
    in >> n >> m >> q;
    Graph g(n);
    for (int i = 0; i < m; i++) {
        in >> a >> b;
        g.add_undirected_edge(a, b);
    }
    vector<pair<int, int>> queries;
    for (int i = 0; i < q; i++) {
        in >> a >> b;
        queries.push_back(make_pair(a, b));
    }
    // costruisco il bridge tree
    Tree bridge_tree;
    g.get_bridge_tree(bridge_tree);
    // trovo un'orientazione agli archi dell'albero
    vector<pair<int, int>> tree_edges_orientation(g.bridges.size(),
                                                  make_pair(-1, -1));
    int conflict_edge = - 1;
    translate_queries(queries, g.bfc);

    if (bridge_tree.answer_queries(queries, tree_edges_orientation,
                                   conflict_edge)) {
        out << "1" << endl;
        // riconverto gli archi dell'albero nei bridges originali
        vector<pair<int, int>> oriented_bridges;
        get_bridges_orientation(tree_edges_orientation, g, oriented_bridges);
        out << oriented_bridges.size() << endl;
        for (auto& bridge : oriented_bridges)
            out << bridge.first << " " << bridge.second << endl;

    } else {
        out << "0" << endl;
        pair<int, int> conflict = g.edges[g.bridges[conflict_edge]];
        out << conflict.first << " " << conflict.second << endl;
    }
}
